@@ -1,7 +1,7 @@
 ;;; upd-copyr.el --- update the copyright notice in a GNU Emacs elisp file
 
-;;; Copyright (C) 1991, 1992 Free Software Foundation, Inc.
-;;; Written by Roland McGrath.
+;;; Copyright (C) 1991-1993 Free Software Foundation, Inc.
+;;; Written by Roland McGrath; hacked on by Jamie Zawinski.
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -68,55 +68,53 @@ than adding to it."
 			 (goto-char (point-min))
 			 (sit-for 0)
 			 (y-or-n-p "Update copyright? ")))))
-	    (progn
-	      (setq replace
-		    (or replace
-			(and ask-year
-			     (save-window-excursion
-			       (pop-to-buffer (current-buffer))
-			       (save-excursion
-				 ;; Show the user the copyright.
-				 (goto-char (point-min))
-				 (sit-for 0)
-				 (y-or-n-p "Replace copyright year? "))))))
-	      (if replace
-		  (delete-region (match-beginning 1) (match-end 1))
-		(insert ", "))
+	    (let ((s (match-beginning 1))
+		  (e (match-end 1)))
+	      (goto-char s)
+	      (cond ((looking-at "[0-9][0-9][0-9][0-9]-")
+		     (goto-char e)
+		     (delete-region (match-end 0) e))
+		    ((looking-at "\\([0-9][0-9][0-9][0-9]\\), *")
+		     (goto-char (match-end 1))
+		     (delete-region (match-end 1) e)
+		     (insert "-"))
+		    (t
+		     (goto-char e)
+		     (insert "-")))
 	      (insert current-year)
-	      (message "Copyright updated to %s%s."
-		       (if replace "" "include ") current-year)
-	  (if replace-copying-with
-	      (let ((case-fold-search t)
-		    beg)
-		(goto-char (point-min))
-		;; Find the beginning of the copyright.
-		(if (search-forward "copyright" nil t)
-		    (progn
-		      ;; Look for a blank line or a line
-		      ;; containing only comment chars.
-		      (if (re-search-forward "^\\(\\s \\s<\\|\\s>\\)*$" nil t)
-			  (forward-line 1)
-			(with-output-to-temp-buffer "*Help*"
-			  (princ (substitute-command-keys "\
+	      (message "Copyright updated to %s." current-year)
+	      (if replace-copying-with
+		  (let ((case-fold-search t)
+			beg)
+		    (goto-char (point-min))
+		    ;; Find the beginning of the copyright.
+		    (if (search-forward "copyright" nil t)
+			(progn
+			  ;; Look for a blank line or a line
+			  ;; containing only comment chars.
+			  (if (re-search-forward "^\\(\\s \\s<\\|\\s>\\)*$" nil t)
+			      (forward-line 1)
+			    (with-output-to-temp-buffer "*Help*"
+			      (princ (substitute-command-keys "\
 I don't know where the copying notice begins.
 Put point there and hit \\[exit-recursive-edit]."))
-			  (recursive-edit)))
-		      (setq beg (point))
-		      (or (search-forward "02139, USA." nil t)
-			  (with-output-to-temp-buffer "*Help*"
-			    (princ (substitute-command-keys "\
+			      (recursive-edit)))
+			  (setq beg (point))
+			  (or (search-forward "02139, USA." nil t)
+			      (with-output-to-temp-buffer "*Help*"
+				(princ (substitute-command-keys "\
 I don't know where the copying notice ends.
 Put point there and hit \\[exit-recursive-edit]."))
-			    (recursive-edit)))
-		      (delete-region beg (point))))
-		(insert-file replace-copying-with))
-	    (if (re-search-forward
-		 "; either version \\(.+\\), or (at your option)"
-		 nil t)
-		(progn
-		  (goto-char (match-beginning 1))
-		  (delete-region (point) (match-end 1))
-		  (insert current-gpl-version)))))
+				(recursive-edit)))
+			  (delete-region beg (point))))
+		    (insert-file replace-copying-with))
+		(if (re-search-forward
+		     "; either version \\(.+\\), or (at your option)"
+		     nil t)
+		    (progn
+		      (goto-char (match-beginning 1))
+		      (delete-region (point) (match-end 1))
+		      (insert current-gpl-version)))))
 	  (or ask-upd
 	      (error "This buffer contains no copyright notice!")))))))
 
