@@ -1,5 +1,5 @@
 ;; Mouse support for X window system.
-;; Copyright (C) 1985, 1987, 1988, 1992 Free Software Foundation, Inc.
+;; Copyright (C) 1985-1993 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -34,7 +34,7 @@ to the cut buffer"
       (x-store-cutbuffer s))
     (kill-region old-point (point))))
 
-(defun x-insert-selection (&optional check-cutbuffer-p)
+(defun x-insert-selection (&optional check-cutbuffer-p move-point-event)
   "Insert the current selection into buffer at point."
   (interactive)
   (let ((text (if check-cutbuffer-p
@@ -42,6 +42,7 @@ to the cut buffer"
 		      (x-get-cutbuffer)
 		      (error "No selection or cut buffer available"))
 		(x-get-selection))))
+    (if move-point-event (mouse-set-point move-point-event))
     (push-mark (point))
     (insert text)
 ;;    (if zmacs-regions (zmacs-activate-region))
@@ -50,8 +51,7 @@ to the cut buffer"
 (defun x-set-point-and-insert-selection (event)
   "Sets point where clicked and insert the primary selection or the cut buffer"
   (interactive "e")
-  (mouse-set-point event)
-  (x-insert-selection t))
+  (x-insert-selection t event))
 
 (defun mouse-track-and-copy-to-cutbuffer (event)
   "Makes a selection like `mouse-track', but also copies it to the cutbuffer."
@@ -90,6 +90,9 @@ If this is nil, then `x-pointer-shape' is used.")
 If this is nil, then either `x-nontext-pointer-shape' or `x-pointer-shape'
 will be used.")
 
+(defvar x-selection-pointer-shape nil
+  "*The shape of the mouse-pointer when over a selectable text region.")
+
 (defvar x-pointer-foreground-color nil
   "*The foreground color of the mouse pointer.")
 
@@ -102,7 +105,9 @@ will be used.")
 	 (buffer (and window (window-buffer window)))
 	 (point (and buffer (event-point event)))
 	 (extent (and point (extent-at point buffer 'highlight)))
-	 (var (cond (point 'x-pointer-shape)
+	 (var (cond ((and extent x-selection-pointer-shape)
+		     'x-selection-pointer-shape)
+		    (point 'x-pointer-shape)
 		    (buffer
 		     (cond (x-nontext-pointer-shape 'x-nontext-pointer-shape)
 			   (x-pointer-shape 'x-pointer-shape)))
@@ -176,6 +181,9 @@ database."
     (setq x-pointer-shape
 	  (or (x-get-resource "textPointer" "Cursor" 'string screen)
 	      "xterm"))
+    (setq x-selection-pointer-shape
+	  (or (x-get-resource "selectionPointer" "Cursor" 'string screen)
+	      "top_left_arrow"))
     (setq x-nontext-pointer-shape
 	  (or (x-get-resource "spacePointer" "Cursor" 'string screen)
 	      "crosshair"))
