@@ -14,8 +14,13 @@ You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#ifndef _EMACS_FACES_H_
+#define _EMACS_FACES_H_
+
 extern void make_face_ready (struct screen* s, struct face* face);
 extern void flush_face_cache ();
 extern void setup_extent_fragment_face_ptr (struct screen *s, EXTENT_FRAGMENT extfrag);
 extern void ensure_face_ready (struct screen* s, int f);
 extern void init_screen_faces (struct screen *s);
+
+#endif /* _EMACS_FACES_H_ */
