@@ -1,11 +1,11 @@
 /* Definitions for interface to indent.c
-   Copyright (C) 1985, 1986 Free Software Foundation, Inc.
+   Copyright (C) 1985-1993 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -17,6 +17,8 @@ You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#ifndef _EMACS_INDENT_H_
+#define _EMACS_INDENT_H_
 
 struct position
   {
@@ -33,3 +35,5 @@ struct position
 
 struct position *compute_motion ();
 struct position *vmotion ();
+
+#endif /* _EMACS_INDENT_H_ */
