@@ -13,12 +13,18 @@
 
 */
 
+#if __STDC__
+#include <stdlib.h>
+#include <unistd.h>
+#endif
 #include <stdio.h>
 #include <fcntl.h>
 #include "libsst.h"
 
 #define AUDBUF 1024
 
+extern void usleep();
+
 int
 sst_open(play_level, record_level)
     int play_level, record_level;
