@@ -1,5 +1,5 @@
 /* Definitions for asynchronous process control in GNU Emacs.
-   Copyright (C) 1985, 1992 Free Software Foundation, Inc.
+   Copyright (C) 1985-1993 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -17,6 +17,8 @@ You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#ifndef _EMACS_PROCESS_H_
+#define _EMACS_PROCESS_H_
 
 /*
  * Structure records pertinent information about open channels.
@@ -78,6 +80,8 @@ struct Lisp_Process
     Lisp_Object update_tick;
 };
 
+#ifdef emacs
+
 #define ChannelMask(n) (1<<(n))
 
 /* True iff we are about to fork off a synchronous process or if we
@@ -91,3 +95,6 @@ extern char *synch_process_death;
    this is exit code of synchronous subprocess.  */
 extern int synch_process_retcode;
 
+#endif /* emacs */
+
+#endif /* _EMACS_PROCESS_H_ */
